/*
 * Sven Kratz, Michael Rohs (c) 2011
 */
package de.lmu.mobilehci.gesturewizards;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;

import android.app.Activity;
import android.gesture.Gesture;
import android.gesture.GestureLibraries;
import android.gesture.GestureLibrary;
import android.gesture.GestureOverlayView;
import android.gesture.Prediction;
import android.gesture.GestureOverlayView.OnGesturePerformedListener;
import android.os.Bundle;
import android.os.Environment;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

/**
 * Reads the gesture library, listens to gesture events, sets up the wizards and
 * the spells, handles pause and resume, and manages the options menu.
 */
public class GestureWizardsActivity extends Activity implements OnGesturePerformedListener {

	public static final String TAG = "GestureWizardsActivity";

	private GestureLibrary gestureLibrary;

	// maps gesture names to spell information objects
	public HashMap<String, SpellInfo> spellInfoMap = new HashMap<String, SpellInfo>();
	
	public boolean gameOver = false;

	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.gestureview);

		Log.d("", "App initialized");

		// set up spells
		spellInfoMap.put("earth", new SpellInfo("earth", Spell.SpellType.Earth,
				R.drawable.testspell1, false, Spell.SpellType.EarthDefense));
		spellInfoMap.put("air", new SpellInfo("air", Spell.SpellType.Air,
				R.drawable.testspell2, false, Spell.SpellType.AirDefense));
		spellInfoMap.put("fire", new SpellInfo("fire", Spell.SpellType.Fire,
				R.drawable.testspell3, false, Spell.SpellType.FireDefense));
		spellInfoMap.put("defearth", new SpellInfo("defearth",
				Spell.SpellType.EarthDefense, R.drawable.defenseearth, true,
				Spell.SpellType.Earth));
		spellInfoMap.put("defair", new SpellInfo("defair",
				Spell.SpellType.AirDefense, R.drawable.defenseair, true,
				Spell.SpellType.Air));
		spellInfoMap.put("deffire", new SpellInfo("deffire",
				Spell.SpellType.FireDefense, R.drawable.defensefire, true,
				Spell.SpellType.Fire));

		// set up wizards
		Wizard.whiteWizard.setContext(this);
		Wizard.blackWizard.setContext(this);
		
		// load gesture library
		File mStoreFile = new File(Environment.getExternalStorageDirectory(),
				"gestures");
		gestureLibrary = GestureLibraries.fromFile(mStoreFile);
		if (!gestureLibrary.load()) {
			Toast.makeText(this, "Keine Gestenbibliothek gefunden. Gesture Builder benutzen!", Toast.LENGTH_LONG).show();
			// finish();
		}

		// set gesture listener
		GestureOverlayView gestures = (GestureOverlayView) findViewById(R.id.gestures);
		gestures.addOnGesturePerformedListener(this);
	}
	
	@Override
	protected void onPause() {
		super.onPause();

		// stop animations
		View wizardView1 = findViewById(R.id.wizard1);
		View wizardView2 = findViewById(R.id.wizard2);
		wizardView1.clearAnimation();
		wizardView2.clearAnimation();
		
		// prevent phone wizard to continue playing...
		Wizard.blackWizard.stopAutomaticWizard();
	}
	
	@Override
	protected void onResume() {
		super.onResume();

		if (!gameOver) {
			Wizard.blackWizard.startAutomaticWizard();
		}
	}

	// from interface OnGesturePerformedListener
	public void onGesturePerformed(GestureOverlayView overlay, Gesture gesture) {
		Log.d(TAG, "OnGesturePerformed");
		
		ArrayList<Prediction> predictions = gestureLibrary.recognize(gesture);

		// the recognizer should have at least one guess (prediction) 
		if (!gameOver && predictions.size() > 0) {
			Prediction prediction = predictions.get(0);
			// the prediction should have some minimal score (the higher the better)
			if (prediction.score > 1.0) {
				
				// Namen der Geste Anzeigen, oder was anderes machen
				Toast.makeText(this, prediction.name, Toast.LENGTH_SHORT).show();

				// check whether we have information about a gesture with this name
				SpellInfo si = spellInfoMap.get(prediction.name);
				if (si != null) {
					int animationResource = (si.defensive) ? 
							R.anim.whitedefspellanima : R.anim.whitespellanima;
					Spell s = Wizard.whiteWizard.castSpell(si, animationResource);
					Wizard.blackWizard.opponentDidLaunchAttackSpell(s);
				}
			}
		}

	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		menu.add(0, 1, 1, "Neustart");
		return super.onCreateOptionsMenu(menu);
	}
	
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		switch (item.getItemId()) {
		case 1: // restart game
			gameOver = false;
			Wizard.whiteWizard.reset();
			Wizard.blackWizard.reset();
			Wizard.blackWizard.startAutomaticWizard();

			// update points text view
			TextView tv = (TextView) findViewById(R.id.gameInfo);
			String output = Integer.toString(Wizard.whiteWizard.points) + " : " + 
					Integer.toString(Wizard.blackWizard.points);
			tv.setText(output);
			break;
		}
		return super.onOptionsItemSelected(item);
	}

}
